package com.smup;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.newdawn.slick.Image;
import org.newdawn.slick.Music;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.Sound;
import org.newdawn.slick.loading.LoadingList;
import org.newdawn.slick.util.Log;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.smup.map.MapPartGenerator;

public class RessourceManager {
	private static RessourceManager _instance = new RessourceManager();
	
	private Map<String, Image> m_images;
	private Map<String, Sound> m_sounds;
	private Map<String, Music> m_musics;
	
	private MapPartGenerator m_mapPartGenerator;
	
	private int totalRessourcesToLoad = 0;
	private int actualRessourceToLoad = 0;
	private NodeList listResources;
	
	private RessourceManager() {
		m_images = new HashMap<String, Image>();
		m_sounds = new HashMap<String, Sound>();
		m_musics = new HashMap<String, Music>();
	}
	
	public final static RessourceManager getInstance() { return _instance; }
	
	public void loadMapRessources() {
		try {
			m_mapPartGenerator = new MapPartGenerator();
		} catch (SlickException e) {
			Log.debug("PB !!");
			e.printStackTrace();
		}
	}
	
	public void loadRessources(InputStream is) throws SlickException {
		loadRessources(is, false);
	}

	private void loadRessources(InputStream is, boolean deferred) throws SlickException {
		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder = null;
		
		try {
			docBuilder = docBuilderFactory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			throw new SlickException("Probleme lors du chargement des ressources !", e);
		}
		
		Document doc = null;
		
		try {
			doc = docBuilder.parse(is);
		} catch (SAXException e) {
			throw new SlickException("Probleme lors du chargement des ressources !", e);
		} catch (IOException e) {
			throw new SlickException("Probleme lors du chargement des ressources !", e);
		}
		
		doc.getDocumentElement().normalize();
		
		listResources = doc.getElementsByTagName("resource");
		int nbRessources = listResources.getLength();
		
		if(deferred)
			LoadingList.setDeferredLoading(true);
		
		totalRessourcesToLoad = nbRessources;
	}
	
	public void loadRessource() throws DOMException, SlickException {
		if(actualRessourceToLoad < totalRessourcesToLoad) {
			Node resourceNode = listResources.item(actualRessourceToLoad);
			
			if(resourceNode.getNodeType() == Node.ELEMENT_NODE) {
				Element resourceElement = (Element)resourceNode;
				
				String type = resourceElement.getAttribute("type");
				
				if(type.equals("image")) {
					addElementAsImage(resourceElement);
				} else if(type.equals("sound")) {
					addElementAsSound(resourceElement);
				} else if(type.equals("music")) {
					addElementAsMusic(resourceElement);
				}
			}
			
			actualRessourceToLoad++;
		}
	}

	private void addElementAsMusic(Element resourceElement) throws DOMException, SlickException {
		loadMusic(resourceElement.getAttribute("id"), resourceElement.getTextContent());
	}

	private void addElementAsSound(Element resourceElement) throws DOMException, SlickException {
		loadSound(resourceElement.getAttribute("id"), resourceElement.getTextContent());
	}

	private void addElementAsImage(Element resourceElement) throws DOMException, SlickException {
		loadImage(resourceElement.getAttribute("id"), resourceElement.getTextContent());
	}
	
	public Music loadMusic(String id, String path) throws SlickException {
		Music mus;
		
		try {
			mus = new Music(path);
		} catch (SlickException e) {
			throw new SlickException("Could not load music " + id + " [" + path + "]", e);
		}
		
		m_musics.put(id, mus);
		Log.debug(" -- New Music : " + id + " [" + path + "]");
		
		return mus;
	}
	
	public Sound loadSound(String id, String path) throws SlickException {
		Sound snd;
		
		try {
			snd = new Sound(path);
		} catch (SlickException e) {
			throw new SlickException("Could not load sound " + id + " [" + path + "]", e);
		}
		
		m_sounds.put(id, snd);
		Log.debug(" -- New Sound : " + id + " [" + path + "]");
		
		return snd;
	}
	
	public Image loadImage(String id, String path) throws SlickException {
		Image img;
		
		try {
			img = new Image(path);
		} catch (SlickException e) {
			throw new SlickException("Could not load image " + id + " [" + path + "]", e);
		}
		
		m_images.put(id, img);
		Log.debug(" -- New Image : " + id + " [" + path + "]");
		
		return img;
	}

	public Image getImageByID(String id) {
		return m_images.get(id);
	}
	
	public Sound getSoundByID(String id) {
		return m_sounds.get(id);
	}
	
	public Music getMusicByID(String id) {
		return m_musics.get(id);
	}

	public MapPartGenerator getMapPartGenerator() {
		return m_mapPartGenerator;
	}

	public int getActualRessourcesLoad() {
		return actualRessourceToLoad;
	}

	public float getTotalRessourcesToLoad() {
		return totalRessourcesToLoad;
	}
	
	public Element getElementToLoad() {
		Node resourceNode = listResources.item(actualRessourceToLoad);
		
		if(resourceNode.getNodeType() == Node.ELEMENT_NODE) {
			return (Element)resourceNode;
		} else {
			return null;
		}
	}
	
}
