package com.smup.map;

import java.util.Enumeration;
import java.util.Hashtable;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.state.StateBasedGame;

import com.smup.BasicStruct;
import com.smup.RessourceManager;
import com.smup.entity.Bullet;
import com.smup.entity.BulletExplosion;
import com.smup.entity.Entity;
import com.smup.ships.Vaisseau;
import com.smup.states.Map;

/**
 *
 * @author Bastien
 */
public class MapPart implements BasicStruct {
	
	protected Map m_map;
	protected String m_nom;
	protected float m_distanceCommencement;
	protected float m_offset = 0;
	protected Image m_sprite;
	protected StateBasedGame m_sbg;
	protected GameContainer m_gc;
	protected Color m_bColor;
	
	protected Hashtable<Integer, Entity> m_entity;
	protected int m_idLastEntity = 0;
	protected boolean m_noClip = false;
	
	public MapPart(Map map, String nom, Color backColor) {
		m_map = map;
		m_sbg = map.getStateBasedGame();
		m_nom = nom;
		m_bColor = backColor;
		m_distanceCommencement = map.getDistanceMax();
		
		m_entity = new Hashtable<Integer, Entity>();
		
		m_sprite = RessourceManager.getInstance().getImageByID(nom);
	}

	@Override
	public void render(GameContainer gc, StateBasedGame sbg, Graphics g) {
		g.setColor(m_bColor);
		g.fillRect(m_offset, 0, m_sprite.getWidth(), 500);
		
		m_sprite.draw(m_offset, 0);
		
		// On dessine les entits
		Enumeration<Entity> entity = m_entity.elements();
		
		while(entity.hasMoreElements()) {
			entity.nextElement().render(gc, sbg, g);
		}
	}

	@Override
	public void update(GameContainer gc, StateBasedGame sbg, int t) {
		// On update les entit�s
		Enumeration<Entity> entity = m_entity.elements();
		
		while(entity.hasMoreElements()) {
			entity.nextElement().update(gc, sbg, t);
		}
	}

	public boolean isBetween(float distanceDepart, float distanceFin) {
		if(m_distanceCommencement > distanceFin || m_distanceCommencement + m_sprite.getWidth() < distanceDepart)
			return false;
		else
			return true;
	}

	public void checkCollisions(float distanceActuelle) {
		// On calcule le d�calage entre la fenetre et le d�but de la partie
		majOffset(distanceActuelle);

		// On v�rifie si le joueur est en collision avec la map
		Vaisseau joueur = m_map.getVaisseauJoueur();
		Shape hb = joueur.getHitbox();
		if(hb.getX() - m_offset > 0 && hb.getX() + hb.getWidth() - m_offset < m_sprite.getWidth() && !m_noClip) {
			if(m_sprite.getColor((int)(hb.getX() - m_offset), (int)hb.getY()).getAlpha() != 0 || 
			   m_sprite.getColor((int)(hb.getX() - m_offset), (int)hb.getY() + (int)hb.getHeight()).getAlpha() != 0 ||	
			   m_sprite.getColor((int)(hb.getX() + hb.getWidth() - m_offset), (int)hb.getY()).getAlpha() != 0 ||
			   m_sprite.getColor((int)(hb.getX() + hb.getWidth() - m_offset), (int)hb.getY() + (int)hb.getHeight()).getAlpha() != 0) {
				joueur.takeDamage(9999); 
			}
		}
		
		// On v�rifie si les balles sont en collision avec le d�cor
		Hashtable<Integer, Bullet> HTbullets = m_map.getBullets();
		
		Enumeration<Bullet> bullets = HTbullets.elements();
		
		while(bullets.hasMoreElements() && !m_noClip) {
			Bullet e = bullets.nextElement();
			if(e.getX() - m_offset > 0 && e.getX() + hb.getWidth() - m_offset < m_sprite.getWidth()) {
				if(m_sprite.getColor((int)(e.getX() - m_offset), (int)e.getY()).getAlpha() != 0) {
					addEntity(new BulletExplosion(this, e.getX() - m_offset - 15, e.getY() - 15));
					m_map.removeBullet(e.getID());
				}
			}
		}
		
		// On regarde si le joueur est en collision avec les entit�s et on maj les offsets
		Enumeration<Entity> entity = m_entity.elements();
		
		while(entity.hasMoreElements()) {
			Entity e = entity.nextElement();
			e.setOffset(m_offset);
			
			if(e.getHitbox().intersects(joueur.getHitbox())) {
				e.onPlayerCollision(joueur);
			}
		}
		
		// Enfin on regarde si les balles du joueur sont en colisions avec une des entit�s
		bullets = HTbullets.elements();
		
		while(bullets.hasMoreElements()) {
			Bullet b = bullets.nextElement();
			if(b.getID() < 0) {
				entity = m_entity.elements();
				
				while(entity.hasMoreElements()) {
					Entity e = entity.nextElement();

					if(e.getHitbox().contains(b.getX(), b.getY()) && e.isDeadly()) {
						e.takeDamage(b.getDegats());
						addEntity(new BulletExplosion(this, b.getX() - m_offset - 15, b.getY() - 15));
						m_map.removeBullet(b.getID());
					}
				}
			}
		}
	}
	
	public void majOffset(float distanceActuelle) {
		m_offset = m_distanceCommencement - distanceActuelle;
	}

	public void removeEntity(int id) {
		m_entity.remove(id);
	}

	public int getWidth() {
		return m_sprite.getWidth();
	}
	
	public void addEntity(Entity e) {
		m_entity.put(++m_idLastEntity, e);
		e.setID(m_idLastEntity);
	}

	public void setNoclip(boolean noClip) {
		m_noClip  = noClip;
	}
}
