package com.smup.map;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Random;

import org.newdawn.slick.Color;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.util.Log;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.smup.RessourceManager;
import com.smup.ships.FoeShip;
import com.smup.states.Map;

public class Zone {
	private String m_nom;
	private String m_lastPartGiven;
	private Random m_rd = new Random();
	private Color m_backgroundColor = Color.black;
	
	private Hashtable<String, String> m_typePart;
	private Hashtable<Integer, ArrayList<String>> m_entryType;
	private Hashtable<String, Integer> m_exitType;
	private Hashtable<String, Boolean> m_noClip;
	private Hashtable<String, Float> m_tempsPause;
	private Hashtable<String, ArrayList<Element>> m_ships;
	
	public Zone(Element zoneElement) {
		m_exitType = new Hashtable<String, Integer>();
		m_typePart = new Hashtable<String, String>();
		m_tempsPause = new Hashtable<String, Float>();
		m_noClip = new Hashtable<String, Boolean>();
		m_ships = new Hashtable<String, ArrayList<Element>>();
		m_entryType = new Hashtable<Integer, ArrayList<String>>();
		
		m_nom = zoneElement.getAttribute("id");
		
		String backColor = zoneElement.getAttribute("backgroundColor");
		
		if(backColor != null) {
			if(backColor.equals("white")) {
				m_backgroundColor = Color.white;
			}
		}
		
		NodeList partList = zoneElement.getElementsByTagName("part");
		
		for(int i = 0; i < partList.getLength(); i++) {
			Element part = (Element)partList.item(i);
			
			Log.debug("Nouvelle partie de la zone : " + part.getAttribute("id"));
			
			// On charge l'image de la partie
			try {
				RessourceManager.getInstance().loadImage(m_nom + "-" + part.getAttribute("id"), 
						"data/graphics/map/" + m_nom + "/" + part.getAttribute("id") + ".png");
			} catch (SlickException e) {
				Log.debug(" -*- ERREUR : L'image de la carte " + m_nom + "-" + part.getAttribute("id") + " n'as pas t charge !");
			}
			
			String type = part.getAttribute("type");
			
			if(type == null || type.equals("")) {
				type = "normal";
			} else if(type.equals("scrollPausingOnEntry")) {
				Log.debug(String.valueOf(Float.parseFloat(part.getAttribute("tempsPause"))));
				m_tempsPause.put(part.getAttribute("id"), Float.parseFloat(part.getAttribute("tempsPause")));
			}
			
			String noClip = part.getAttribute("noClip");
			
			if(noClip != null) {
				if(noClip.equals("1")) {
					m_noClip.put(part.getAttribute("id"), true);
				} else {
					m_noClip.put(part.getAttribute("id"), false);
				}
			} else {
				m_noClip.put(part.getAttribute("id"), false);
			}
			
			m_typePart.put(part.getAttribute("id"), type);
			
			NodeList nextPart = part.getElementsByTagName("nextPart");
			
			int typeSortie = -9999;
			
			if(nextPart.getLength() > 0) {
				typeSortie = Integer.valueOf(((Element)(nextPart.item(0))).getTextContent());
			}
			
			m_exitType.put(part.getAttribute("id"), typeSortie);
			
			// On note les entres
			NodeList entryTypes = part.getElementsByTagName("entryType");
			
			int entryType = -1;
			
			for(int j = 0; j < entryTypes.getLength(); j++) {
				entryType = Integer.valueOf(((Element)(entryTypes.item(j))).getTextContent());
				
				if(m_entryType.get(entryType) != null) {
					m_entryType.get(entryType).add(part.getAttribute("id"));
				} else {
					ArrayList<String> entryList =  new ArrayList<String>();
					entryList.add(part.getAttribute("id"));
					m_entryType.put(entryType, entryList);
				}
			}
			
			NodeList foeList = part.getElementsByTagName("foe");
			ArrayList<Element> foeArray = new ArrayList<Element>();
			
			for(int k = 0; k < foeList.getLength(); k++) {
				Element foe = (Element)foeList.item(k);
				foeArray.add(foe);
			}
			
			m_ships.put(part.getAttribute("id"), foeArray);
		}
		
		Log.debug("Nouvelle zone id = " + m_nom);
	}
	
	public MapPart getMapPart(Map map) {
		// Si on a jamais donn de partie de cette zone
		if(m_lastPartGiven == null) {
			return getMapPart(map, "entry");
		} else { // Sinon on en gnre un alatoirement
			int typeSortie = m_exitType.get(m_lastPartGiven);
			
			if(typeSortie == -9999) {
				// SI c'est la sortie (si il n'y a aucune prochainne map, on remet le lastPartGiven  null et on renvoi null
				m_lastPartGiven = null;
				return null;
			} else { // Sinon on gnre une nouvelle parties
				return getMapPart(map, m_exitType.get(m_lastPartGiven));
			}
		}
	}
	
	private MapPart getMapPart(Map map, int typeEntree) {		
		boolean invalide = true;
		String nextMap = null;
		
		while(invalide) {
			nextMap = m_entryType.get(typeEntree).get(m_rd.nextInt(m_entryType.get(typeEntree).size()));
			
			if(!nextMap.equals(m_lastPartGiven)) 
				invalide = false;
		}
		
		return getMapPart(map, nextMap);
	}

	private MapPart getMapPart(Map map, String name) {
		MapPart mp = null;
		
		String type = m_typePart.get(name);
		
		if(type.equals("normal")) {
			mp = new MapPart(map, m_nom + "-" + name, m_backgroundColor);
		} else if(type.equals("scrollPausingOnEntry")) {
			Log.debug("special");
			mp = new ScrollPausingOnEntryMapPart(map, m_nom + "-" + name, m_backgroundColor, m_tempsPause.get(name));
		}
		
		mp.setNoclip(m_noClip.get(name));

		// Si ya des ships  mettre c'est le moment !
		ArrayList<Element> ships = m_ships.get(name);
		
		for(int i = 0; i < ships.size(); i++) {
			Element ship = ships.get(i);
			String shipType = ship.getAttribute("type");
			
			if(shipType.equals("tourelle")) {
				mp.addEntity(new FoeShip(map, mp, 
						Float.valueOf(ship.getAttribute("x")),
						Float.valueOf(ship.getAttribute("y")),
						Integer.valueOf(ship.getAttribute("delayTir")),
						Float.valueOf(ship.getAttribute("precision"))));
			}
		}
		
		Log.debug("ON DONNE LA CARTE : " + name + " / " + type + " / " + mp);
		m_lastPartGiven = name;
		return mp;
	}

	public void reset() {
		m_lastPartGiven = null;
	}
}
