package com.smup.ships;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.state.StateBasedGame;

import com.smup.BasicStruct;
import com.smup.entity.Entity;
import com.smup.sprite.Sprite;
import com.smup.states.Map;

public abstract class Ship extends Entity implements BasicStruct {
	protected Sprite m_sprite;
	protected Shape m_hitbox;
	protected float m_vitesse;
	protected Map m_map;
	protected boolean m_peuxTirer = false;
	protected int m_delayTir;
	protected int m_tempsDepuisTir = 0;
	
	public Ship(Map map, float vitesse, int delayTir, String path) {
		m_canDie = true;
		
		try {
			m_sprite = new Sprite(path);
			m_sprite.setFilter(0);
			refreshHitbox();
		} catch (SlickException e) {
			e.printStackTrace();
		}
		
		m_map = map;
		m_vitesse = vitesse;
		m_delayTir = delayTir;
	}
	
	@Override
	public void render(GameContainer gc, StateBasedGame sbg, Graphics g) {
		m_sprite.draw(m_sprite.getX() + m_offset, m_sprite.getY());
	}

	@Override
	public void update(GameContainer gc, StateBasedGame sbg, int t) {
		updateDelayTir(t);
	}
	
	public void refreshHitbox() {
		if(m_hitbox == null) {
			m_hitbox = new Rectangle(m_sprite.getX(), m_sprite.getY(), m_sprite.getWidth(), m_sprite.getHeight());
		} else {
			m_hitbox.setX(m_sprite.getX() + m_offset);
			m_hitbox.setY(m_sprite.getY());
		}
	}
	
	public void setVitesse(float v) { m_vitesse = v; }
	public void setDelayTir(int d) { m_delayTir = d; }
	
	public void updateDelayTir(int t) {
		if((m_tempsDepuisTir += t) > m_delayTir) {
			m_peuxTirer = true;
		}
	}
	
	public float getX() { return m_sprite.getX(); }
	public float getY() { return m_sprite.getY(); }
	public Shape getHitbox() { return m_hitbox; }
	
	public void move(float x, float y) {
		m_sprite.move(x, y);
		refreshHitbox();		
	}
	
	public abstract void tirer();
	
	public float getFireRate() {
		return 1 / (0.001f * m_delayTir);
	}
	
	public void setFireRate(float rate) {
		m_delayTir = (int) ((1 / rate) * 1000);
	}
}
