package com.smup.ships;

import java.util.ArrayList;

import org.newdawn.slick.Animation;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.Sound;
import org.newdawn.slick.SpriteSheet;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.state.StateBasedGame;

import com.smup.RessourceManager;
import com.smup.buffs.Buff;
import com.smup.buffs.BuffManager;
import com.smup.buffs.Incendie;
import com.smup.buffs.Regeneration;
import com.smup.buffs.ShootRateModificator;
import com.smup.entity.Bullet;
import com.smup.states.Map;

public class Vaisseau extends Ship {
	
	private Sound m_sonTir;
	private float m_precision = 0.9f;
	private Animation m_animMort;
	BuffManager m_buffs;
	
	private Image m_contourVie;
	private Image m_contenueVie;
	private Animation m_degatsVaisseau;

	public Vaisseau(Map map, float x, float y) {
		super(map, 200.f, 350, "vaisseau");
		m_sprite.setPosition(x, y);
		refreshHitbox();
		
		RessourceManager rm = RessourceManager.getInstance();
		
		m_sonTir = rm.getSoundByID("shoot1");
		m_buffs = new BuffManager();
		setFireRate(2.5f);
		
		m_contourVie = rm.getImageByID("vieContour_hud");
		m_contenueVie = rm.getImageByID("vie_hud");
		
		m_degatsVaisseau = new Animation(new SpriteSheet(RessourceManager.getInstance().getImageByID("degatsPlayer"), 20, 21),
				 0, 0, 2, 0, true, 50, true);
		m_degatsVaisseau.setLooping(false);
		
		m_animMort = new Animation(new SpriteSheet(RessourceManager.getInstance().getImageByID("playerExplosion"), 64, 64),
				 0, 0, 16, 0, true, 50, true);
		m_animMort.stop();
		m_animMort.setLooping(false);
	}

	@Override
	public void update(GameContainer gc, StateBasedGame sbg, int t) {
		super.update(gc, sbg, t);
		
		Input in = gc.getInput();
		
		if(in.isKeyDown(Input.KEY_LEFT) || in.isKeyDown(Input.KEY_Q)) {
			move(-m_vitesse * 0.001f * t, 0);
		}
		if(in.isKeyDown(Input.KEY_RIGHT) || in.isKeyDown(Input.KEY_D)) {
			move(m_vitesse * 0.001f * t, 0);
		}
		if(in.isKeyDown(Input.KEY_UP) || in.isKeyDown(Input.KEY_Z)) {
			move(0, -m_vitesse * 0.001f * t);
		}
		if(in.isKeyDown(Input.KEY_DOWN) || in.isKeyDown(Input.KEY_S)) {
			move(0, m_vitesse * 0.001f * t);
		}
		if(in.isKeyDown(Input.KEY_SPACE) && m_peuxTirer) {
			tirer();
		}
		if(in.isKeyPressed(Input.KEY_A)) {
			m_buffs.add(new Incendie(this, 15, 1));
		}
		if(in.isKeyPressed(Input.KEY_E)) {
			//m_delayTir += 10;
			m_buffs.add(new Regeneration(this, 15, 1));
		}
		if(in.isKeyPressed(Input.KEY_R)) {
			m_buffs.add(new ShootRateModificator(this, 20, 2));
		}
		if(in.isKeyPressed(Input.KEY_ADD)) {
			m_vitesse = 50.f;
		}
		if(in.isKeyPressed(Input.KEY_SUBTRACT)) {
			m_vitesse = 250.f;
		}
		
		m_buffs.update(t);
	}	
	
	@Override
	public void render(GameContainer gc, StateBasedGame sbg, Graphics g) {
		if(m_vie > 0) {
			m_sprite.draw();
			
			if(!m_degatsVaisseau.isStopped())
				m_degatsVaisseau.draw(m_sprite.getX(), m_sprite.getY());
			
			m_buffs.render(g);
		}
		else if(!m_animMort.isStopped())
			m_animMort.draw(m_sprite.getX() - 15, m_sprite.getY() - 25);

		m_contourVie.draw(10,  10);
		m_contenueVie.getSubImage(0, 0, (int)(m_contenueVie.getWidth() * (m_vie / (float)(m_vieMax))), m_contenueVie.getHeight()).draw(11, 11);
		
		g.setColor(new Color(230, 230, 230));
		g.drawString(m_vie + " PV", 85, 9);
		
		ArrayList<Image> icons = m_buffs.getIcons();
		ArrayList<Float> durations = m_buffs.getDurations();
		
		for(int i = 0; i < icons.size(); i++) {
			icons.get(i).draw(i * 37 + 215, 10);

			g.drawString(String.valueOf(Math.round(durations.get(i))) + "s", i * 37 + 215, 45);
		}
	}
	
	@Override
	public void refreshHitbox() {
		if(m_hitbox == null) {
			m_hitbox = new Rectangle(m_sprite.getX() + 2, m_sprite.getY() + 7, 8, 7);
		} else {
			m_hitbox.setX(m_sprite.getX() + 2);
			m_hitbox.setY(m_sprite.getY() + 7);
		}
	}
	
	public void tirer() {
		if(!m_sonTir.playing())
			m_sonTir.play();
		m_map.addBullet(new Bullet(m_sprite.getX() + m_sprite.getWidth(), 
								   m_sprite.getY() + m_sprite.getHeight() / 2, 
								   600.f, 0.f, m_map.getVitesseScroll(), m_precision, 150), true);
		m_peuxTirer = !m_peuxTirer;
		m_tempsDepuisTir = 0;
	}

	@Override
	protected void onDeath() {
		m_vie = 0;
		
		m_animMort.restart();
		
		m_map.gameOver();
	}

	public void deleteBuff(Buff buff) {
		m_buffs.delete(buff);
	}
	
	@Override
	public void takeDamage(int amount) {
		super.takeDamage(amount);
		
		if(amount > 0)
			m_degatsVaisseau.restart();
	}

	public void addBuff(Buff b) {
		m_buffs.add(b);
	}

}
