package com.smup.states;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

import com.smup.RessourceManager;
import com.smup.entity.Bullet;
import com.smup.map.MapPart;
import com.smup.map.MapPartGenerator;
import com.smup.ships.Vaisseau;

public class Map extends BasicGameState {
	
	public static final int ID = 1;   
	
	private StateBasedGame m_sbg;
	private GameContainer m_gc;
	private Vaisseau m_vaisseau;	
	private Hashtable<Integer, Bullet> m_bullets; // id des Balles tires par le joueur < 0
	private int m_idLastBulletJoueur = 0;
	private int m_idLastBulletOther = 0;
	
	private MapPartGenerator m_mapPartGenerator;
	private LinkedList<MapPart> m_partiesMap;
	private float m_distanceActuelle = 0;
	private float m_distanceMax = 0;
	private float m_vitesseScrolling = 150.f;
	private float m_tempsPauseScrolling = 0.f;
	
	public Map() {
		m_vaisseau = new Vaisseau(this, 50, 250);
		
		m_bullets = new Hashtable<Integer, Bullet>();
		m_partiesMap = new LinkedList<MapPart>();
		
		m_mapPartGenerator = RessourceManager.getInstance().getMapPartGenerator();
	}
	
	@Override
	public void init(GameContainer gc, StateBasedGame sbg)
			throws SlickException {
		m_sbg = sbg;
		m_gc = gc;
		
		while(m_distanceMax < m_gc.getWidth())
			addMapPart();
	}
	
	private void addMapPart() {
		addMapPart(m_mapPartGenerator.getMapPart(this));
	}
	
	private void addMapPart(MapPart mp) {
		m_partiesMap.add(mp);
		m_distanceMax += mp.getWidth();
		mp.majOffset(m_distanceActuelle);
	}

	@Override
	public void render(GameContainer gc, StateBasedGame sbg, Graphics g)
			throws SlickException {
		/*g.setColor(Color.black);
		g.fillRect(0, 0, gc.getWidth(), gc.getHeight());*/
		
		// On dessine les mapparts background
		// Pour le moment, on se contente de dessiner la map normalement
		for(int i = 0; i < m_partiesMap.size(); i++) {
			MapPart mp = m_partiesMap.get(i);
			
			if(mp.isBetween(m_distanceActuelle, m_distanceActuelle + gc.getWidth()))
				mp.render(gc, sbg, g);
		}
		
		// On dessine les balles
		Enumeration<Bullet> bullets = m_bullets.elements();
		
		while(bullets.hasMoreElements()) {
			bullets.nextElement().render(gc, sbg, g);
		}
		
		// On dessine le vaisseau
		m_vaisseau.render(gc, sbg, g);
		
		// On dessine les mapparts foreground
		
		// On affiche la distance parcourue, les fps, etc.
		g.setColor(new Color(230, 230, 230));
		g.drawString("Distance : " + String.valueOf(m_distanceActuelle / 100.f + 0.001).substring(0,  5) + " km", 20, 25);
		
		g.setColor(Color.red);
		g.drawString("FPS : " + m_gc.getFPS(), 900, 10);
	}

	@Override
	public void update(GameContainer gc, StateBasedGame sbg, int t)
			throws SlickException {
		// On update la distance actuelle
		if(m_tempsPauseScrolling <= 0) {
			m_distanceActuelle += m_vitesseScrolling * 0.001f * t;
			if(m_distanceMax - 200 < m_distanceActuelle + m_gc.getWidth())
				addMapPart();
		} else
			m_tempsPauseScrolling -= 0.001f * t; 
		
		// On update le vaisseau
		m_vaisseau.update(gc, sbg, t);
		
		// On update les balles
		Enumeration<Bullet> bullets = m_bullets.elements();
		
		while(bullets.hasMoreElements()) {
			Bullet e = bullets.nextElement();
			e.update(gc, sbg, t);
		}
		
		// On update les parties 
		for(int i = 0; i < m_partiesMap.size(); i++) {
			MapPart mp = m_partiesMap.get(i);
			
			if(mp.isBetween(m_distanceActuelle, m_distanceActuelle + gc.getWidth()))
				mp.update(gc, sbg, t);
		}
		
		// On vrifie les collisions, etc...
		checkGameLogic();
	}
	
	private void checkGameLogic() {
		// On vrifie que le joueur soit bien dans l'cran
		Shape hitboxJ = m_vaisseau.getHitbox();
		if(hitboxJ.getX() < 0)
			m_vaisseau.move(-hitboxJ.getX(), 0);
		if(hitboxJ.getY() < 0)
			m_vaisseau.move(0, -hitboxJ.getY());
		if(hitboxJ.getY() + hitboxJ.getHeight() > 500)
			m_vaisseau.move(0, -(hitboxJ.getY() + hitboxJ.getHeight() - 500));
		if(hitboxJ.getX() + hitboxJ.getWidth() > 1000)
			m_vaisseau.move(-(hitboxJ.getX() + hitboxJ.getWidth() - 1000), 0);
		
		// On vrifie si les balles sont toujours dans l'cran
		Enumeration<Bullet> bullets = m_bullets.elements();
		
		while(bullets.hasMoreElements()) {
			Bullet e = bullets.nextElement();
			int xBullet = (int) e.getX(),
				yBullet = (int) e.getY();
			if(xBullet > m_gc.getWidth() || xBullet < 0 || yBullet > m_gc.getHeight() || yBullet < 0) {
				removeBullet(e.getID());
			}
		}
		
		// On regarde dans quel partie de la map on est
		for(int i = 0; i < m_partiesMap.size(); i++) {
			MapPart mp = m_partiesMap.get(i);
			
			if(mp.isBetween(m_distanceActuelle, m_distanceActuelle + m_gc.getWidth())) {
				// On check les collisions avec le joueur et les bullets
				mp.checkCollisions(m_distanceActuelle);
			}
		}
		
		// On regarde si les balles enemies touchent le joueur
		bullets = m_bullets.elements();
		
		while(bullets.hasMoreElements()) {
			Bullet b = bullets.nextElement();
			if(b.getID() > 0) {
				if(m_vaisseau.getHitbox().contains(b.getX(), b.getY())) {
					m_vaisseau.takeDamage(b.getDegats());
					removeBullet(b.getID());
				}
			}
		}
	}

	public void removeBullet(int id) {
		m_bullets.remove(id);
	}

	public void addBullet(Bullet b, boolean tirDuJoueur) {
		int idBullet;
		
		if(tirDuJoueur)
			idBullet = --m_idLastBulletJoueur;
		else
			idBullet = ++m_idLastBulletOther;
			
		m_bullets.put(idBullet, b);
		b.setID(idBullet);
	}

	@Override
	public int getID() {
		return ID;
	}

	public Vaisseau getVaisseauJoueur() {
		return m_vaisseau;
	}

	public int getCoupsTiresParJoueur() {
		return -1 * m_idLastBulletJoueur;
	}

	public Hashtable<Integer, Bullet> getBullets() {
		return m_bullets;
	}
	
	public void invert() { 
		m_vitesseScrolling *= -1;
		updateVitesseScrolling();
	}

	private void updateVitesseScrolling() {
		Enumeration<Bullet> bullets = m_bullets.elements();
		
		while(bullets.hasMoreElements()) {
			bullets.nextElement().setVitesseScroll(m_vitesseScrolling);
		}
	}

	public float getVitesseScroll() {
		return m_vitesseScrolling; 
	}
	
	public void gameOver() {
		RessourceManager rm = RessourceManager.getInstance();
		
		rm.getSoundByID("gameOverSound").play();
		
		m_sbg.addState(new GameOver(getCoupsTiresParJoueur()));
		
		try {
			m_sbg.getState(GameOver.ID).init(m_gc, m_sbg);
		} catch (SlickException ex) {
			ex.printStackTrace();
		}
		
		rm.getMusicByID("bgm").stop();
		
		m_sbg.enterState(GameOver.ID, new FadeOutTransition(new Color(0, 0, 0, 0), 1500), new FadeInTransition(new Color(0, 0, 0, 0), 1000));
	}

	public void pauseScrolling(float t) {
		m_tempsPauseScrolling += t;
	}

	public StateBasedGame getStateBasedGame() {
		return m_sbg;
	}

	public float getDistanceMax() {
		return m_distanceMax;
	}
	
	@Override
	public void keyPressed(int code, char key) {
		if(code == Input.KEY_F1) {
			m_gc.setVSync(!m_gc.isVSyncRequested());
		} else if((code == Input.KEY_ESCAPE) || (code == Input.KEY_P)) {
			RessourceManager.getInstance().getMusicByID("bgm").pause();
			m_sbg.enterState(PauseMenu.ID, new FadeOutTransition(), new FadeInTransition());
		}
	}
	
	public void reset() {
		RessourceManager.getInstance().getMapPartGenerator().reset();
		
		m_distanceActuelle = 0;
		m_distanceMax = 0;
		m_vitesseScrolling = 150.f;
		m_tempsPauseScrolling = 0.f;
		
		m_vaisseau = new Vaisseau(this, 50, 250);
		
		m_bullets = new Hashtable<Integer, Bullet>();
		m_partiesMap = new LinkedList<MapPart>();
		
		m_mapPartGenerator = RessourceManager.getInstance().getMapPartGenerator();
		m_idLastBulletJoueur = 0;
		m_idLastBulletOther = 0;
		
		while(m_distanceMax < m_gc.getWidth())
			addMapPart();
	}
}
